import os
from collections.abc import AsyncIterator
from contextlib import asynccontextmanager
from typing import List, TypedDict

from mcp.server.fastmcp import FastMCP

if os.environ.get("TESTING"):
    from tests.fake_immich_api import ImmichAPI
else:
    from immich_mcp.immich_api import ImmichAPI


class User(TypedDict):
    """Represents a user in Immich."""

    id: str
    email: str
    name: str


UsersList = List[User]


class Partner(User):
    """Represents a partner in Immich."""

    inTimeline: bool


PartnersList = List[Partner]


class Asset(TypedDict):
    """Represents an asset in Immich."""

    id: str
    originalFileName: str
    type: str


class ApiKey(TypedDict):
    """Represents an API key in Immich."""

    id: str
    name: str
    createdAt: str
    updatedAt: str
    permissions: List[str]


ApiKeyList = List[ApiKey]


class AppContext(TypedDict):
    """Application context holding shared resources."""

    immich_client: ImmichAPI


@asynccontextmanager
async def app_lifespan(server: FastMCP) -> AsyncIterator[AppContext]:
    """Manage the application's lifespan, creating and cleaning up resources."""
    print("Initializing app lifespan")
    async with ImmichAPI() as immich_client:
        print("ImmichAPI client created")
        yield AppContext(immich_client=immich_client)
    print("App lifespan finished")


mcp = FastMCP(name="ImmichMCP", lifespan=app_lifespan)


@mcp.tool()
async def ping() -> str:
    """
    Pings the real Immich server to check for a valid connection.
    Returns 'pong' if successful, otherwise returns an error message.
    """
    ctx = mcp.get_context()
    immich_client = ctx.request_context.lifespan_context["immich_client"]
    if await immich_client.ping_server():
        return "pong"
    return "error: could not connect to Immich server"


@mcp.resource("user://me")
async def get_user() -> User | None:
    """Returns the current user's details."""
    ctx = mcp.get_context()
    immich_client = ctx.request_context.lifespan_context["immich_client"]
    user_data = await immich_client.get_my_user()
    if not user_data:
        raise ValueError("Failed to fetch user from Immich API")
    return User(id=user_data["id"], email=user_data["email"], name=user_data["name"])


@mcp.resource("users://list")
async def get_users_list() -> UsersList:
    """Returns a list of all users."""
    ctx = mcp.get_context()
    immich_client = ctx.request_context.lifespan_context["immich_client"]
    users_data = await immich_client.get_users_list()
    return [User(id=user["id"], email=user["email"], name=user["name"]) for user in users_data]


@mcp.resource("partners://list")
async def get_partners() -> PartnersList:
    """Returns a list of all partners."""
    ctx = mcp.get_context()
    immich_client = ctx.request_context.lifespan_context["immich_client"]
    partners_data = await immich_client.get_partners()
    return [
        Partner(
            id=partner["id"],
            email=partner["email"],
            name=partner["name"],
            inTimeline=partner["inTimeline"],
        )
        for partner in partners_data
    ]


@mcp.resource("asset://{asset_id}")
async def get_asset(asset_id: str) -> Asset | None:
    """Returns an asset by its ID."""
    ctx = mcp.get_context()
    immich_client = ctx.request_context.lifespan_context["immich_client"]
    asset_data = await immich_client.get_asset(asset_id)
    if not asset_data:
        return None
    return Asset(
        id=asset_data["id"],
        originalFileName=asset_data["originalFileName"],
        type=asset_data["type"],
    )


@mcp.resource("apikey://me")
async def get_my_api_key() -> ApiKey | None:
    """Returns the current API key's details."""
    ctx = mcp.get_context()
    immich_client = ctx.request_context.lifespan_context["immich_client"]
    api_key_data = await immich_client.get_my_api_key()
    if not api_key_data:
        return None
    return ApiKey(
        id=api_key_data["id"],
        name=api_key_data["name"],
        createdAt=api_key_data["createdAt"],
        updatedAt=api_key_data["updatedAt"],
        permissions=api_key_data["permissions"],
    )


@mcp.resource("apikeys://list")
async def get_api_key_list() -> ApiKeyList:
    """Returns a list of all API keys."""
    ctx = mcp.get_context()
    immich_client = ctx.request_context.lifespan_context["immich_client"]
    api_keys_data = await immich_client.get_api_key_list()
    return [
        ApiKey(
            id=key["id"],
            name=key["name"],
            createdAt=key["createdAt"],
            updatedAt=key["updatedAt"],
            permissions=key["permissions"],
        )
        for key in api_keys_data
    ]


@mcp.resource("apikey://{api_key_id}")
async def get_api_key(api_key_id: str) -> ApiKey | None:
    """Returns an API key by its ID."""
    ctx = mcp.get_context()
    immich_client = ctx.request_context.lifespan_context["immich_client"]
    api_key_data = await immich_client.get_api_key(api_key_id)
    if not api_key_data:
        return None
    return ApiKey(
        id=api_key_data["id"],
        name=api_key_data["name"],
        createdAt=api_key_data["createdAt"],
        updatedAt=api_key_data["updatedAt"],
        permissions=api_key_data["permissions"],
    )


if __name__ == "__main__":
    mcp.run(transport="stdio")



import time

from httpx import AsyncClient


async def initialize_session(client: AsyncClient) -> str:
    """Perform the MCP initialize handshake and return a session ID."""
    init_request = {
        "jsonrpc": "2.0",
        "method": "initialize",
        "params": {
            "protocolVersion": "2025-06-18",
            "clientInfo": {"name": "immich-mcp-test-client", "version": "0.1.0"},
            "capabilities": {},
        },
        "id": "init1",
    }
    headers = {"Accept": "application/json, text/event-stream"}
    response = await client.post("/mcp", json=init_request, headers=headers)
    response.raise_for_status()
    time.sleep(0.1)
    session_id = response.headers["mcp-session-id"]
    # Complete the initialization handshake with the correct notification
    init_notify = {
        "jsonrpc": "2.0",
        "method": "notifications/initialized",
        "params": {},
    }
    notify_headers = {
        "Accept": "application/json, text/event-stream",
        "mcp-session-id": session_id,
    }
    notify_resp = await client.post("/mcp", json=init_notify, headers=notify_headers)
    notify_resp.raise_for_status()
    return session_id



from unittest.mock import patch

import pytest
from httpx import AsyncClient
from starlette.responses import JSONResponse


# A dummy class to patch
class MyClass:
    def __init__(self):
        pass


# A dummy ASGI app
async def app(scope, receive, send):
    response = JSONResponse({"hello": "world"})
    await response(scope, receive, send)


@pytest.mark.asyncio
async def test_httpx_with_patch():
    with patch("tests.temp_test_httpx.MyClass.__init__", return_value=None):
        async with AsyncClient(app=app, base_url="http://test") as aclient:
            response = await aclient.get("/")
            assert response.status_code == 200
            assert response.json() == {"hello": "world"}



from mcp.server.fastmcp import FastMCP

from immich_mcp.server import mcp


def test_server_instance_exists():
    """Tests that the FastMCP server instance is created."""
    assert isinstance(mcp, FastMCP)
    assert mcp.name == "ImmichMCP"



import asyncio
import json
import os
import subprocess
import sys
import time

from pydantic_ai.mcp import MCPServerStreamableHTTP


async def main():
    """
    Main function to run the functional tests using the pydantic-ai client.
    This is the final, final version.
    """
    api_url = os.environ.get("IMMICH_BASE_URL")
    api_key = os.environ.get("IMMICH_API_KEY")

    if not api_url or not api_key:
        print("Error: IMMICH_BASE_URL and IMMICH_API_KEY must be set.", file=sys.stderr)
        sys.exit(1)

    server_host = "127.0.0.1"
    server_port = 8765
    server_address = f"http://{server_host}:{server_port}/mcp"

    server_command = [
        "python",
        "-m",
        "uvicorn",
        "src.immich_mcp.main:app",
        "--host",
        server_host,
        "--port",
        str(server_port),
    ]

    print("Starting server...")
    server_process = subprocess.Popen(
        server_command,
        stdout=sys.stdout,
        stderr=sys.stderr,
        env={**os.environ, "PYTHONPATH": "src"},
    )

    try:
        # Give the server time to start up
        time.sleep(10)

        # Create an MCP client server instance
        server = MCPServerStreamableHTTP(url=server_address)

        async with server:
            print("Client session initialized.")

            client_session = server._client

            # --- Run Tests ---
            print("\n--- Testing ping ---")
            ping_result = await client_session.call_tool("ping", {})
            ping_text = ping_result.content[0].text
            print(f"Response: {ping_text}")
            assert ping_text == "pong", f"Ping failed: {ping_text}"
            print("PASS: Ping successful.")

            print("\n--- Testing user://me ---")
            user_result = await client_session.read_resource("user://me")
            user = json.loads(user_result.contents[0].text)
            print(f"Response: {user}")
            assert user.get("id"), "User response did not contain an ID."
            print(f"PASS: Got user with ID: {user['id']}")

            print("\n--- Testing users://list ---")
            users_list_result = await client_session.read_resource("users://list")
            users_list = json.loads(users_list_result.contents[0].text)
            print(f"Response: {users_list}")
            assert isinstance(users_list, list) and len(users_list) > 0, "Did not get a valid list of users."
            print(f"PASS: Got a list of {len(users_list)} users.")

            print("\n--- Testing partners://list ---")
            partners_list_result = await client_session.read_resource("partners://list")
            partners_list = json.loads(partners_list_result.contents[0].text)
            print(f"Response: {partners_list}")
            assert isinstance(partners_list, list), "Partners list was not a valid list."
            print(f"PASS: Got a list of {len(partners_list)} partners.")

            print("\n--- Testing apikey://me ---")
            my_api_key_result = await client_session.read_resource("apikey://me")
            my_api_key = json.loads(my_api_key_result.contents[0].text)
            print(f"Response: {my_api_key}")
            assert my_api_key.get("id"), "My API key response did not contain an ID."
            print(f"PASS: Got current API key with ID: {my_api_key['id']}")

            print("\n--- Testing apikeys://list and apikey://{id} ---")
            api_keys_list_result = await client_session.read_resource("apikeys://list")
            api_keys_list = json.loads(api_keys_list_result.contents[0].text)
            print(f"List Response: {api_keys_list}")
            assert isinstance(api_keys_list, list) and len(api_keys_list) > 0, (
                "Did not get a valid list of API keys."
            )

            first_key_id = api_keys_list[0].get("id")
            assert first_key_id, "First API key in list has no ID."
            print(f"Found API key ID to test: {first_key_id}")

            specific_key_result = await client_session.read_resource(f"apikey://{first_key_id}")
            specific_key = json.loads(specific_key_result.contents[0].text)
            print(f"Specific Key Response: {specific_key}")
            assert specific_key.get("id") == first_key_id, "Fetched specific key ID does not match."
            print(f"PASS: Successfully fetched specific API key with ID: {first_key_id}")

        print("\nAll functional tests passed!")

    except Exception as e:
        print(f"\nAn error occurred during testing: {e}", file=sys.stderr)
        sys.exit(1)

    finally:
        print("\nCleaning up and stopping the server...")
        server_process.terminate()
        server_process.wait()
        print("Server stopped.")


if __name__ == "__main__":
    asyncio.run(main())



import json
import os
import threading
from unittest.mock import AsyncMock, patch

import pytest
import pytest_asyncio
import uvicorn
from httpx import AsyncClient

from immich_mcp.immich_api import ImmichAPI
from immich_mcp.server import mcp
from tests.utils import initialize_session


class UvicornTestServer(uvicorn.Server):
    """Uvicorn test server that runs in a thread."""

    def __init__(self, app, host="127.0.0.1", port=8000):
        self._startup_done = threading.Event()
        config = uvicorn.Config(app, host=host, port=port, log_level="info")
        super().__init__(config)

    def run_in_thread(self):
        self.thread = threading.Thread(target=self.run)
        self.thread.start()
        self._startup_done.wait(timeout=5)
        if not self._startup_done.is_set():
            raise RuntimeError("Server did not start in time.")

    def stop(self):
        self.should_exit = True
        self.thread.join(timeout=5)
        if self.thread.is_alive():
            raise RuntimeError("Server did not stop in time.")

    async def startup(self, sockets=None):
        await super().startup(sockets=sockets)
        self._startup_done.set()


@pytest_asyncio.fixture(scope="module")
async def server():
    """Fixture to run the MCP server in a background thread."""
    app = mcp.streamable_http_app()
    server = UvicornTestServer(app)
    server.run_in_thread()
    yield f"http://{server.config.host}:{server.config.port}"
    server.stop()


# `initialize_session` moved to `tests/utils.py` and imported above.


@pytest.mark.asyncio
@patch.dict(os.environ, {"IMMICH_BASE_URL": "http://test.com", "IMMICH_API_KEY": "test-key"})
@patch(
    "immich_mcp.immich_api.ImmichAPI.ping_server",
    new_callable=AsyncMock,
    return_value=True,
)
async def test_ping_tool_success(mock_ping_server, server: str):
    """Tests the ping tool against a running server."""
    async with AsyncClient(base_url=server) as client:
        session_id = await initialize_session(client)

        tool_request = {
            "jsonrpc": "2.0",
            "method": "tools/call",
            "params": {"name": "ping", "arguments": {}},
            "id": "1",
        }
        headers = {
            "Accept": "application/json, text/event-stream",
            "mcp-session-id": session_id,
        }

        async with client.stream("POST", "/mcp", json=tool_request, headers=headers) as response:
            response.raise_for_status()
            sse_event = await response.aiter_bytes().__anext__()
            data_part = sse_event.split(b"data: ")[1].strip()
            tool_result = json.loads(data_part)

            assert tool_result["id"] == "1"
            assert "result" in tool_result
            assert tool_result["result"]["content"][0]["text"] == "pong"


@pytest.mark.asyncio
@patch.dict(os.environ, {"IMMICH_BASE_URL": "http://test.com", "IMMICH_API_KEY": "test-key"})
@patch(
    "immich_mcp.immich_api.ImmichAPI.ping_server",
    new_callable=AsyncMock,
    return_value=False,
)
async def test_ping_tool_failure(mock_ping_server, server: str):
    """Tests the ping tool with a mocked failed connection."""
    async with AsyncClient(base_url=server) as client:
        session_id = await initialize_session(client)

        tool_request = {
            "jsonrpc": "2.0",
            "method": "tools/call",
            "params": {"name": "ping", "arguments": {}},
            "id": "2",
        }
        headers = {
            "Accept": "application/json, text/event-stream",
            "mcp-session-id": session_id,
        }
        async with client.stream("POST", "/mcp", json=tool_request, headers=headers) as response:
            response.raise_for_status()
            sse_event = await response.aiter_bytes().__anext__()
            data_part = sse_event.split(b"data: ")[1].strip()
            tool_result = json.loads(data_part)

            assert tool_result["id"] == "2"
            assert "result" in tool_result
            assert "error: could not connect" in tool_result["result"]["content"][0]["text"]


def test_immich_api_raises_on_missing_env_vars():
    """
    Tests that the ImmichAPI class raises a ValueError if the required
    environment variables are not set.
    """
    with patch.dict(os.environ, {}, clear=True):
        with pytest.raises(ValueError, match="Immich base URL must be provided"):
            ImmichAPI()



import json
import os

import httpx


class ImmichAPI:
    """A client for interacting with the Immich API."""

    def __init__(self, base_url: str | None = None, api_key: str | None = None):
        self.base_url = base_url or os.environ.get("IMMICH_BASE_URL")
        self.api_key = api_key or os.environ.get("IMMICH_API_KEY")

        if not self.base_url:
            raise ValueError(
                "Immich base URL must be provided via argument or IMMICH_BASE_URL environment variable."
            )
        if not self.api_key:
            raise ValueError(
                "Immich API key must be provided via argument or IMMICH_API_KEY environment variable."
            )

        # Ensure the base URL does not end with a slash, then append /api
        api_url = f"{self.base_url.rstrip('/')}/api"

        self._client = httpx.AsyncClient(
            base_url=api_url,
            headers={
                "x-api-key": self.api_key,
                "Accept": "application/json",
            },
            timeout=30.0,
        )

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()

    async def close(self):
        """Closes the HTTP client."""
        await self._client.aclose()

    async def ping_server(self) -> bool:
        """Pings the Immich server to check for a valid connection."""
        try:
            response = await self._client.get("/server/ping")
            response.raise_for_status()
            data = response.json()
            return bool(data and data.get("res") == "pong")
        except (httpx.RequestError, httpx.HTTPStatusError, json.JSONDecodeError, ValueError):
            return False
        except Exception:
            return False

    async def get_my_user(self) -> dict:
        """Fetches the current user's details."""
        try:
            response = await self._client.get("/users/me")
            response.raise_for_status()
            return response.json()
        except (httpx.RequestError, httpx.HTTPStatusError, json.JSONDecodeError):
            return {}

    async def get_users_list(self) -> list[dict]:
        """Fetches the list of users."""
        try:
            response = await self._client.get("/users")
            response.raise_for_status()
            return response.json()
        except (httpx.RequestError, httpx.HTTPStatusError, json.JSONDecodeError):
            return []

    async def get_partners(self) -> list[dict]:
        """Fetches the list of partners."""
        try:
            response = await self._client.get("/partners", params={"direction": "shared-by"})
            response.raise_for_status()
            return response.json()
        except (httpx.RequestError, httpx.HTTPStatusError, json.JSONDecodeError):
            return []

    async def get_asset(self, asset_id: str) -> dict:
        """Fetches a single asset by its ID."""
        try:
            response = await self._client.get(f"/assets/{asset_id}")
            response.raise_for_status()
            return response.json()
        except (httpx.RequestError, httpx.HTTPStatusError, json.JSONDecodeError):
            return {}

    async def get_my_api_key(self) -> dict:
        """Fetches the current API key's details."""
        try:
            response = await self._client.get("/api-keys/me")
            response.raise_for_status()
            return response.json()
        except (httpx.RequestError, httpx.HTTPStatusError, json.JSONDecodeError):
            return {}

    async def get_api_key_list(self) -> list[dict]:
        """Fetches the list of API keys."""
        try:
            response = await self._client.get("/api-keys")
            response.raise_for_status()
            return response.json()
        except (httpx.RequestError, httpx.HTTPStatusError, json.JSONDecodeError):
            return []

    async def get_api_key(self, api_key_id: str) -> dict:
        """Fetches a single API key by its ID."""
        try:
            response = await self._client.get(f"/api-keys/{api_key_id}")
            response.raise_for_status()
            return response.json()
        except (httpx.RequestError, httpx.HTTPStatusError, json.JSONDecodeError):
            return {}



class ImmichAPI:
    """A fake client for interacting with the Immich API."""

    def __init__(self, api_url: str | None = None, api_key: str | None = None):
        pass

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    async def close(self):
        pass

    async def ping_server(self) -> bool:
        return True

    async def get_my_user(self) -> dict:
        return {
            "id": "test-user-id",
            "email": "test@example.com",
            "name": "Test User",
        }

    async def get_users_list(self) -> list[dict]:
        return [
            {"id": "user1", "email": "user1@example.com", "name": "User One"},
            {"id": "user2", "email": "user2@example.com", "name": "User Two"},
        ]

    async def get_partners(self) -> list[dict]:
        return [
            {
                "id": "partner1",
                "email": "partner1@example.com",
                "name": "Partner One",
                "inTimeline": True,
            },
            {
                "id": "partner2",
                "email": "partner2@example.com",
                "name": "Partner Two",
                "inTimeline": False,
            },
        ]

    async def get_asset(self, asset_id: str) -> dict:
        return {
            "id": asset_id,
            "originalFileName": "test.jpg",
            "type": "IMAGE",
        }

    async def get_my_api_key(self) -> dict:
        return {
            "id": "api-key-1",
            "name": "My API Key",
        }

    async def get_api_key_list(self) -> list[dict]:
        return [
            {"id": "api-key-1", "name": "API Key 1"},
            {"id": "api-key-2", "name": "API Key 2"},
        ]

    async def get_api_key(self, api_key_id: str) -> dict:
        return {
            "id": api_key_id,
            "name": "API Key 1",
        }



from unittest.mock import AsyncMock

import pytest
import pytest_asyncio
from pytest_mock import MockerFixture

from immich_mcp.server import (
    get_api_key,
    get_api_key_list,
    get_asset,
    get_my_api_key,
    get_partners,
    get_user,
    get_users_list,
)


@pytest_asyncio.fixture
def mock_mcp_context(mocker: MockerFixture):
    """Provides a mock MCP context with a mocked ImmichAPI client."""
    mock_context = mocker.patch("immich_mcp.server.mcp.get_context")
    mock_api_client = AsyncMock()
    mock_context.return_value.request_context.lifespan_context = {"immich_client": mock_api_client}
    mock_context.return_value.api_key = "my-fake-api-key"
    return mock_context


@pytest.mark.asyncio
async def test_get_user_resource(mock_mcp_context):
    """Tests that the user resource correctly returns a mock user."""
    mock_api_client = mock_mcp_context.return_value.request_context.lifespan_context["immich_client"]
    mock_api_client.get_my_user.return_value = {
        "id": "test-user-id",
        "email": "test@example.com",
        "name": "Test User",
    }

    user = await get_user()

    assert user is not None
    assert user["id"] == "test-user-id"
    assert user["email"] == "test@example.com"
    assert user["name"] == "Test User"
    mock_api_client.get_my_user.assert_awaited_once()


@pytest.mark.asyncio
async def test_get_users_list_resource(mock_mcp_context):
    """Tests that the users resource correctly returns a mock list of users."""
    mock_api_client = mock_mcp_context.return_value.request_context.lifespan_context["immich_client"]
    mock_api_client.get_users_list.return_value = [
        {"id": "user1", "email": "user1@example.com", "name": "User One"},
        {"id": "user2", "email": "user2@example.com", "name": "User Two"},
    ]

    users_list = await get_users_list()

    assert len(users_list) == 2
    assert users_list[0]["name"] == "User One"
    assert users_list[1]["email"] == "user2@example.com"
    mock_api_client.get_users_list.assert_awaited_once()


@pytest.mark.asyncio
async def test_get_partners_resource(mock_mcp_context):
    """Tests that the partners resource correctly returns a mock list of partners."""
    mock_api_client = mock_mcp_context.return_value.request_context.lifespan_context["immich_client"]
    mock_api_client.get_partners.return_value = [
        {
            "id": "partner1",
            "email": "partner1@example.com",
            "name": "Partner One",
            "inTimeline": True,
        },
        {
            "id": "partner2",
            "email": "partner2@example.com",
            "name": "Partner Two",
            "inTimeline": False,
        },
    ]
    partners_list = await get_partners()

    assert len(partners_list) == 2
    assert partners_list[0]["name"] == "Partner One"
    assert partners_list[0]["inTimeline"] is True
    assert partners_list[1]["email"] == "partner2@example.com"
    assert partners_list[1]["inTimeline"] is False
    mock_api_client.get_partners.assert_awaited_once()


@pytest.mark.asyncio
async def test_get_asset_resource(mock_mcp_context):
    """Tests that the asset resource correctly returns a mock asset."""
    mock_api_client = mock_mcp_context.return_value.request_context.lifespan_context["immich_client"]
    mock_api_client.get_asset.return_value = {
        "id": "asset1",
        "originalFileName": "test.jpg",
        "type": "IMAGE",
    }
    asset = await get_asset(asset_id="asset1")

    assert asset is not None
    assert asset["id"] == "asset1"
    assert asset["originalFileName"] == "test.jpg"
    assert asset["type"] == "IMAGE"
    mock_api_client.get_asset.assert_awaited_once_with("asset1")


@pytest.mark.asyncio
async def test_get_my_api_key_resource(mock_mcp_context):
    """Tests that the my_api_key resource correctly returns the current api key."""
    mock_api_client = mock_mcp_context.return_value.request_context.lifespan_context["immich_client"]
    mock_api_client.get_my_api_key.return_value = {
        "id": "api-key-me",
        "name": "My API Key",
        "createdAt": "2025-09-02T00:00:00Z",
        "updatedAt": "2025-09-02T00:00:00Z",
        "permissions": ["all"],
    }
    api_key = await get_my_api_key()

    assert api_key is not None
    assert api_key["id"] == "api-key-me"
    mock_api_client.get_my_api_key.assert_awaited_once()


@pytest.mark.asyncio
async def test_get_api_key_list_resource(mock_mcp_context):
    """Tests that the api_keys resource correctly returns a mock list of api keys."""
    mock_api_client = mock_mcp_context.return_value.request_context.lifespan_context["immich_client"]
    mock_api_client.get_api_key_list.return_value = [
        {
            "id": "api-key-1",
            "name": "API Key 1",
            "createdAt": "2025-09-02T00:00:00Z",
            "updatedAt": "2025-09-02T00:00:00Z",
            "permissions": ["all"],
        },
        {
            "id": "api-key-2",
            "name": "API Key 2",
            "createdAt": "2025-09-02T00:00:00Z",
            "updatedAt": "2025-09-02T00:00:00Z",
            "permissions": ["all"],
        },
    ]
    api_key_list = await get_api_key_list()

    assert len(api_key_list) == 2
    assert api_key_list[0]["name"] == "API Key 1"
    assert api_key_list[1]["id"] == "api-key-2"
    mock_api_client.get_api_key_list.assert_awaited_once()


@pytest.mark.asyncio
async def test_get_api_key_resource(mock_mcp_context):
    """Tests that the api_key resource correctly returns a mock api key."""
    mock_api_client = mock_mcp_context.return_value.request_context.lifespan_context["immich_client"]
    mock_api_client.get_api_key.return_value = {
        "id": "api-key-1",
        "name": "API Key 1",
        "createdAt": "2025-09-02T00:00:00Z",
        "updatedAt": "2025-09-02T00:00:00Z",
        "permissions": ["all"],
    }
    api_key = await get_api_key(api_key_id="api-key-1")

    assert api_key is not None
    assert api_key["id"] == "api-key-1"
    assert api_key["name"] == "API Key 1"
    mock_api_client.get_api_key.assert_awaited_once_with("api-key-1")


@pytest.mark.asyncio
async def test_get_user_resource_error(mock_mcp_context):
    """Tests that the user resource correctly raises an error."""
    mock_api_client = mock_mcp_context.return_value.request_context.lifespan_context["immich_client"]
    mock_api_client.get_my_user.return_value = {}  # Simulate empty response

    with pytest.raises(ValueError) as excinfo:
        await get_user()

    assert "Failed to fetch user from Immich API" in str(excinfo.value)
    mock_api_client.get_my_user.assert_awaited_once()
